/************************************************************************/
/*																		*/
/*	PmodTMPdriver.c	--	Contains functions used to drive the PmodTMP.	*/
/*																		*/
/************************************************************************/
/*	Author: 	Mark Taylor												*/
/*	Copyright 2008, Digilent Inc.										*/

/************************************************************************/
/*  Revision History:													*/
/*																		*/
/*	03/20/2008(MarkT): created											*/
/*																		*/
/************************************************************************/

/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */

#include <avr/io.h>
#include <stdlib.h>
#include <stdio.h>
#include <inttypes.h>


#include "stdtypes.h"
#include "CerebotNano.h"

/* ------------------------------------------------------------ */
/*				Local Type Definitions							*/
/* ------------------------------------------------------------ */

/* 	Pin out declarations for the PmodTMP.
	These should be editted to be compatible with the DDRx, PORTx,
	and PINx registers that you are using.  The listed definitions here
	are compatible with the Digilent CerebotII board using port B of the 
	CerebotII.*/
#define ddrPmodTMPrst	ddrJB7
#define ddrPmodTMPdq	ddrJB8
#define ddrPmodTMPclk	ddrJB10
#define prtPmodTMPrst	prtJB7
#define prtPmodTMPdq	prtJB8
#define prtPmodTMPclk	prtJB10
#define pinPmodTMPrst	pinJB7
#define pinPmodTMPdq	pinJB8
#define pinPmodTMPclk	pinJB10
#define bnPmodTMPrst	bnJB7
#define bnPmodTMPdq		bnJB8
#define bnPmodTMPclk	bnJB10

/* 	Dallas Semi. DS1626/DS1726 command set.  This
	command set is explained in detail in the Dallas data sheet. */
#define cmdStartConvertT	0x51
#define cmdStopConvertT		0x22
#define cmdReadTemperature	0xAA
#define cmdReadTH			0xA1
#define cmdReadTL			0xA2
#define cmdWriteTH			0x01
#define cmdWriteTL			0x02
#define cmdReadConfig		0xAC
#define cmdWriteConfig		0x0C
#define cmdPOR				0x54

/*	Desired Configuration Register setup. 
	See the Dallas Semi DS1626/DS1726 data sheet for
	additional information about the Configuration Register.*/
					//one-shot, no stand-alone, 9-bit resolution
#define bConfigData			0b00000011

/*	Bit position in the Configuration Register of the DONE bit */
#define posDONE				7


/* ------------------------------------------------------------ */
/*				Global Variables								*/
/* ------------------------------------------------------------ */



/* ------------------------------------------------------------ */
/*				Local Variables									*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Forward Declarations							*/
/* ------------------------------------------------------------ */


/* Wait_ms() is defined in my main.c file.  Cut the following and 
place it in your main.c file: 

void Wait_ms(WORD delay) {
	WORD i;

	while(delay > 0){

		for( i = 0; i < 390; i ++){  //i < 390 assumes a clock of 8MHz
			;;
		}
		delay -= 1;
	}
}
*/

//tells the AVR Studio compiler that this function is located elsewhere
extern void	Wait_ms(WORD ms); 

//These are the functions used to drive the PmodTMP
void		InitPmodTMP(void);
int16_t		GetTemperature(void);


/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/***	InitPmodTMP
**
**	Synopsis:
**		InitPmodTMP()
**
**	Parameters:
**		none
**
**	Return Values:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This routine sets up the PmodTMP to simply report the ambient
**		temperature in 1Shot Mode using the DS1626 with 12-bit precision.
**
**		DS1626 can not be clocked any faster than 1.75 MHz.  Device needs
**		100ns between rising RST edge and falling CLK edge.  Device needs
**		a minimum of 40ns to read the DQ line after the rising edge of CLK.
**		Device needs minimum of 35ns before clock's rising edge to set up 
**		DQ line.
**
**		We will simply put data and commands on the DQ line on the falling edge
**		of the clock and read data on the DQ line on the rising edge.
**
**		In this routine, we will set up the DS1626 as follows:
**			R1		= 1:	see R0
**			R0		= 1:	12-bit conversion mode (may take up to 750 ms to read!)
**			CPU 	= 1: 	Stand-alone mode is disabled
**			1SHOT 	= 1:	One-Shot Mode.  The cmdStartConvertT command
**							initiates a single temperature conversion and then 
**							the device goes into a low-power standby state.
**	
*/

void InitPmodTMP(void) {	

	BYTE i;

/*	Set up pins that connect to the Digilent PmodTMP board.
	This code should be moved to the function where you set up all of
	your boards IO pins.*/
	prtPmodTMPrst	&= ~(1 << bnPmodTMPrst);  //LLV
	prtPmodTMPdq	&= ~(1 << bnPmodTMPdq);
	prtPmodTMPclk	&= ~(1 << bnPmodTMPclk);

	ddrPmodTMPrst	|= (1 << bnPmodTMPrst);  //initially all outputs
	ddrPmodTMPdq	|= (1 << bnPmodTMPdq);
	ddrPmodTMPclk	|= (1 << bnPmodTMPclk);


	/* bring RST line high to enable PmodTMP */
	prtPmodTMPrst |= (1 << bnPmodTMPrst);

/*	Start clocking out write config. command*/
	for(i = 0; i < 8; i ++) {
		//lower clock
		prtPmodTMPclk &= ~(1 << bnPmodTMPclk); //falling edge of clock
		
		//set prtPmodTMPdq based on state of data bit in position i 
		if(cmdWriteConfig & (1 << i)) {
			prtPmodTMPdq |= (1 << bnPmodTMPdq); //set dq if data bit set
		}
		else {
			prtPmodTMPdq &= ~(1 << bnPmodTMPdq); //otherwise clear dq
		}
				
		prtPmodTMPclk |= (1 << bnPmodTMPclk); //rising edge of clock
	}

/* 	Start writing Configuration Reg. Data */
	for(i = 0; i < 8; i ++) {
		//lower clock
		prtPmodTMPclk &= ~(1 << bnPmodTMPclk); //falling edge of clock
		
		//set prtPmodTMPdq based on state of data bit in position i 
		if(bConfigData & (1 << i)) {
			prtPmodTMPdq |= (1 << bnPmodTMPdq); //set dq if data bit set
		}
		else {
			prtPmodTMPdq &= ~(1 << bnPmodTMPdq); //otherwise clear dq
		}
			
		prtPmodTMPclk |= (1 << bnPmodTMPclk); //rising edge of clock
	}

	//make sure DQ pin is tied to ground
	prtPmodTMPdq &= ~(1 << bnPmodTMPdq); //LLV

	//disable device by returning RST line low
	prtPmodTMPrst &= ~(1 << bnPmodTMPrst);
}

/* ------------------------------------------------------------ */
/***	GetTemperature
**
**	Synopsis:
**		int16_t GetTemperature(void)
**
**	Parameters:
**		none
**
**	Return Values:
**		wTemperature, signed 16-bit integer
**
**	Errors:
**		none
**
**	Description:
**		This routine requests that the PmodTMP takes a temperature
**		reading.  It then waits, polling the DONE bit in the Configuration
**		register until the conversion is complete.  It then clocks out the 
**		temperature data and returns it.
**	
*/
int16_t GetTemperature(void) {	

	BYTE i;
	BYTE bRegister;
	int16_t wTemperature; //signed integer

	bRegister = 0;
	wTemperature = 0;


	/* bring RST line high to enable PmodTMP */
	prtPmodTMPrst |= (1 << bnPmodTMPrst);


/*	Start clocking out StartConvertT command*/
	for(i = 0; i < 8; i ++) {
		//lower clock
		prtPmodTMPclk &= ~(1 << bnPmodTMPclk); //falling edge of clock
		
		//set prtPmodTMPdq based on state of data bit in position i 
		if(cmdStartConvertT & (1 << i)) {
			prtPmodTMPdq |= (1 << bnPmodTMPdq); //set dq if data bit set
		}
		else {
			prtPmodTMPdq &= ~(1 << bnPmodTMPdq); //otherwise clear dq
		}		
		prtPmodTMPclk |= (1 << bnPmodTMPclk); //rising edge of clock
	}

	/*	End command and disable device by returning RST line low.  You must
		end each command/read with the device by toggling the RST line. */
	prtPmodTMPrst &= ~(1 << bnPmodTMPrst);


/*	Poll DONE bit in Configuration Register until the conversion is complete. */
	do {

		/* Toggle Led1 to indicate we are waiting for conversion to complete.
			This LED is located on the Minicon.  Remove this line if you do not
			want visual indicator of the conversion process or if you do not
			have a spare LED on your board. */
		//prtLed1 ^= (1 << bnLed1); 
		
		/* Do the following every 10ms.
			The following function is defined in my main.c file and is a simple 
			while loop designed to take a WORD input and loop for the specified
			number of milliseconds before returning. */
		Wait_ms(10);

		/* bring RST line high to enable PmodTMP */
		prtPmodTMPrst |= (1 << bnPmodTMPrst);


	/* Clock out command to read Configuration Register */
		for(i = 0; i < 8; i ++) {
			//lower clock
			prtPmodTMPclk &= ~(1 << bnPmodTMPclk); //falling edge of clock
		
			//set prtPmodTMPdq based on state of data bit in position i 
			if(cmdReadConfig & (1 << i)) {
				prtPmodTMPdq |= (1 << bnPmodTMPdq); //set dq if data bit set
			}
			else {
				prtPmodTMPdq &= ~(1 << bnPmodTMPdq); //otherwise clear dq
			}		

			prtPmodTMPclk |= (1 << bnPmodTMPclk); //rising edge of clock
		}
	

		// Set DQ pin to read response of device	
		prtPmodTMPdq &= ~(1 << bnPmodTMPdq); //no pull-ups
		ddrPmodTMPdq &= ~(1 << bnPmodTMPdq); //input


	/* Clock in Configuration Register data */
		for(i = 0; i < 8; i ++) {
			//lower clock
			prtPmodTMPclk &= ~(1 << bnPmodTMPclk); //falling edge of clock
		
			/* Ensure device is given time to write bit before rising edge.
				Depending on the speed of your clock, these assembly "nop" commands
				may be totally unnecessary.  Check the DS1626/DS1726 data sheet for
				the minimum rise/fall times for the device. */
			asm volatile("nop");
			asm volatile("nop");
				
			prtPmodTMPclk |= (1 << bnPmodTMPclk); //rising edge of clock

			if( pinPmodTMPdq & (1 << bnPmodTMPdq) ) { //if data bit high
				bRegister |= (1 << i); //set bit in variable
			}
			else {
				bRegister &= ~(1 << i); //otherwise clear bit in variable
			}

		}
		
		//end command and disable device by returning RST line low
		prtPmodTMPrst &= ~(1 << bnPmodTMPrst);

		ddrPmodTMPdq |= (1 << bnPmodTMPdq); //output again

	} while ( ! (bRegister & (1 << posDONE) ) ); //while DONE bit is not set 


	/* Turn off Led1 to indicate we are done waiting for conversion to complete.  
		Remove this line if you are not using an LED as a visual indicator of
		conversion time */
	//prtLed1 &= ~(1 << bnLed1); 

	
	 //give it a bit of time before issuing next command
	Wait_ms(2);


	/* bring RST line high to enable PmodTMP */
	prtPmodTMPrst |= (1 << bnPmodTMPrst);

/* Clock out command to read Temperature Register */
	for(i = 0; i < 8; i ++) {
		//lower clock
		prtPmodTMPclk &= ~(1 << bnPmodTMPclk); //falling edge of clock
	
		//set prtPmodTMPdq based on state of data bit in position i 
		if(cmdReadTemperature & (1 << i)) {
			prtPmodTMPdq |= (1 << bnPmodTMPdq); //set dq if data bit set
		}
		else {
			prtPmodTMPdq &= ~(1 << bnPmodTMPdq); //otherwise clear dq
		}		

		prtPmodTMPclk |= (1 << bnPmodTMPclk); //rising edge of clock
	}


	// Set DQ pin to read output of device	
	prtPmodTMPdq &= ~(1 << bnPmodTMPdq); //no pull-ups
	ddrPmodTMPdq &= ~(1 << bnPmodTMPdq); //input


/* Clock in Temperature Register data */
	for(i = 0; i < 12; i ++) { //Temperature data is 12-bits long
		//lower clock
		prtPmodTMPclk &= ~(1 << bnPmodTMPclk); //falling edge of clock
	
		//Ensure device is given time to write bit before rising edge
		asm volatile("nop");
		asm volatile("nop");
			
		prtPmodTMPclk |= (1 << bnPmodTMPclk); //rising edge of clock

		if( pinPmodTMPdq & (1 << bnPmodTMPdq) ) { //if data bit high
			wTemperature |= (1 << i); //set bit in variable
		}
	}

	//check sign bit and convert to signed 16-bit (12-bit result to 16-bit int)
	if(wTemperature & (1 << 11) ) {
		for(i = 12; i < 16; i++) {
			wTemperature |= (1 << i); //set
		}
	}
	
	//end command and disable device by returning RST line low
	prtPmodTMPrst &= ~(1 << bnPmodTMPrst);
	
	ddrPmodTMPdq |= (1 << bnPmodTMPdq); //output again
	
/*	wTemperature currently reflects the contents of the device's Temperature
	Register.  This notation includes binary decimal units of type Celsius.
	In my application, I do not need the additional bits beyond the 
	decimal point, so we will shift them out.  Then convert to Fahrenheit. */

	wTemperature = (wTemperature >> 4); //shift out bits 0 -> 3. Bit 4 becomes LSB.

	//convert to Fahrenheit
	wTemperature = ( ( (wTemperature * 9) / 5) + 32 ); //classic equation

	return wTemperature;
}










