/************************************************************************/
/*																		*/
/*	main.c	--	Main program file for the PmodRF1 reference design		*/
/*																		*/
/************************************************************************/
/*	Author: 	Mark Taylor												*/
/*	Copyright	2009, Digilent Inc.										*/
/************************************************************************/
/*  Module Description: 												*/
/*	This program is meant to be a very low power remote temperature 	*/
/*	measurement probe that takes measurements via the PmodTMP and 		*/
/*	transmits the temperature data to an base unit using the PmodRF1.	*/
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/*		12/09/2009 (MarkT) : created									*/
/*		2/25/2010 (AaronO) : changed display term from "out" to "remote"*/
/*																		*/
/************************************************************************/


/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>


#include	"stdtypes.h"
#include	"CerebotNano.h"
#include	"PmodRF1.h"
#include	"bit_bang_spi.h"
#include	"util.h"
#include	"config.h"


/* ------------------------------------------------------------ */
/*				Local Type Definitions							*/
/* ------------------------------------------------------------ */

//Defines how long the AVR will sleep if driven by timer
#define SLEEPTIME 115 //115 * 261.12ms = ~ 30.029s.

//Voltage divider on pin JB3.  This powers divider,read by ADC6
#define	prtBattery	prtJB3
#define	ddrBattery	ddrJB3
#define	bnBattery	bnJB3

/* ------------------------------------------------------------ */
/*				Global Variables								*/
/* ------------------------------------------------------------ */

volatile BYTE cntSleepTime;
volatile BYTE fBatteryDone;
volatile WORD wBattery;

/* ------------------------------------------------------------ */
/*				Local Variables									*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Macros											*/
/* ------------------------------------------------------------ */

#define		__startTimer2()		TCNT2 = 0;\
								TCCR2B |= (1 << CS22)|(1 << CS21)|(1 << CS20);\
								TIMSK2 |= (1 << TOIE2)
								
#define		__stopTimer2()		TCCR2B &= ~(1 << CS22)|(1 << CS21)|(1 << CS20);\
								TIMSK2 &= ~(1 << TOIE2)

#define		__getBattery()		fBatteryDone = fFalse;\
								ADCSRA |= (1 << ADSC)

/* ------------------------------------------------------------ */
/*				Forward Declarations							*/
/* ------------------------------------------------------------ */

void DeviceInit (void);
void AppInit(void);
void Wait_ms(WORD ms);
BYTE RF1SendString(BYTE *szIn, BYTE cPacketLen);

//Functions located in PmodTMPdriver.c
extern void		InitPmodTMP(void);
extern int16_t	GetTemperature(void);

/* ------------------------------------------------------------ */
/*				Interrupt Service Routines						*/
/* ------------------------------------------------------------ */

SIGNAL(SIG_OVERFLOW2) {	
	cntSleepTime += 1;
}

/* ADC done!  Get result and place in global variables */
SIGNAL ( SIG_ADC) {

 	//Read lower 8 bits of result first.
	wBattery = ADCL;
	//Then shift in upper 2 bits.
	wBattery |= (ADCH << 8);	
	
	fBatteryDone = fTrue;
}
/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------ */

/***	main
**
**	Synopsis:
**		st = main()
**
**	Parameters:
**		none
**
**	Return Values:
**		does not return
**
**	Errors:
**		none
**
**	Description:
**		Main program module. Performs basic board initialization
**		and then enters the main program loop.
*/

int main(void) {
	
	char buf[16];

	
	DeviceInit();
	AppInit();


	//This is the main program loop
	while(fTrue) {

		/********************************/
		/*Battery voltage to ADC:		*/
		/*	3.6V = abs max -> 100%		*/
		/*	2.7V = abs min -> 0%		*/
		/*	Conversion from ADC to %:	*/
		/*	% = (ADC - 750)/2.5			*/
		/*	% = (2)(ADC - 750)/((2)2.5)	*/
		/*	Given: R2 = 20K Ohms		*/
		/*		R1 = 47K Ohms			*/
		/*		VRef = 1.1V				*/
		/*		10-bit ADC max=1023		*/
		/*		30% of battery voltage	*/
		/*			remains at ADC = 825*/
		/*		10% of battery voltage	*/
		/*			remains at ADC = 775*/
		/*		5% of battery voltage	*/
		/*			remains at ADC = 763*/
		/********************************/
		prtBattery |= (1 << bnBattery); //Power voltage divider
		Wait_ms(2); //let voltage settle
		__getBattery();
		while(! fBatteryDone) {
			asm volatile("nop");
		}
		prtBattery &= ~(1 << bnBattery); //LLV to save power
		if(wBattery <= 775) {
			sprintf(buf, "Remote: %3d LowBatt", GetTemperature());
		}
		else { //Good battery left
			sprintf(buf, "Remote: %3d", GetTemperature());
		}
		
		RF1SendString((BYTE*)buf, (BYTE)strlen(buf)); //Send packet via the PmodRF1

		/***************** Sleep Block ******************************************/	
		SMCR = (1 << SM1)|(1 << SM0);  //sets up default sleep mode to "Power Save" mode
		
		//Start sleep timer
		__startTimer2();
		
		SMCR |= (1 << SE);  //enable Sleep to be executed.

		//sleep for a time in power save mode.
		while (cntSleepTime < SLEEPTIME){ 
			asm volatile("SLEEP");  //put device to sleep.
		}

		cntSleepTime = 0; //reset

		//Stop sleep timer
		__stopTimer2();
		/***************** Sleep Block ******************************************/

		
	}  //end while
} //end main()

/* ------------------------------------------------------------ */
/***	DeviceInit
**
**	Synopsis:
**		DeviceInit()
**
**	Parameters:
**		none
**
**	Return Values:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Initializes on chip peripheral devices to the default
**		state.
*/

void DeviceInit() {

	/* Turn all pins as outputs tied to ground.
	*/
	DDRB = 0xFF;
	DDRC = 0xFF;
	DDRD = 0xFF;

	PORTB = 0x00;
	PORTC = 0x00;
	PORTD = 0x00;

	//PmodTMP, pin3 is unused
	ddrJB9 &= ~(1 << bnJB9);
	prtJB9 |= (1 << bnJB9);
	InitPmodTMP(); //initialize the PmodTMP
	
	// Initialize the SPI interfaces.
	BitBangSpiInit();

	//Init additional signals for PmodRF1
	ddrRF1IRQ &= ~(1 << bnRF1IRQ); //input
	prtRF1IRQ &= ~(1 << bnRF1IRQ); //IRQ pin	
	prtRF1RST |= (1 << bnRF1RST);
	ddrRF1RST |= (1 << bnRF1RST); //RST pin	
	prtRF1SLPTR &= ~(1 << bnRF1SLPTR); //must initially be low, only edge sensitive on IC
	ddrRF1SLPTR |= (1 << bnRF1SLPTR); //output, used to enable SLEEP state	
	prtRF1CLKM &= ~(1 << bnRF1CLKM); //CLKM pin floating
	ddrRF1CLKM &= ~(1 << bnRF1CLKM);
	
	//Explicitly declare battery monitor pin
	ddrBattery |= (1 << bnBattery);
	prtBattery &= ~(1 << bnBattery); //LLV
	
} //end DeviceInit()

/* ------------------------------------------------------------ */
/***	AppInit
**
**	Synopsis:
**		AppInit()
**
**	Parameters:
**		none
**
**	Return Values:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Performs application specific initialization. Sets devices
**		and global variables for application.
*/

void AppInit() {

	BYTE rfrTemp;


	//cut power to un-used peripherals
	PRR = (1 << PRTWI)|(1 << PRTIM0)|(1 << PRTIM1)|(1 << PRUSART0)|(1 << PRSPI);

	//init globals
	cntSleepTime = 0;


	/************ Initialize the PmodRF1 ****************/
	RF1Init();

   // Set the channel (center frequency) used for wireless communication.
    rfrTemp = BRF1ReadReg(rfrPhyCcCca);
    rfrTemp &= ~(mskChan);
    rfrTemp |= ( mskChan & rfchan ); // rfchan is defined in config.h
    RF1WriteReg(rfrPhyCcCca, rfrTemp);

    // Set the data rate for the PmodRF1 to 250kbps.
    RF1WriteReg(rfrTrxCtrl2, 0x0c);

    // Clear the PmodRF1 interrupt status register.
    BRF1ReadReg(rfrIrqStatus);

    // Enable PmodRF1 interrupt mask mode (polling).
    rfrTemp = BRF1ReadReg(rfrTrxCtrl1);
    rfrTemp |= ( 1 << bnIrqMaskMode );
    RF1WriteReg(rfrTrxCtrl1, rfrTemp);

    // Enable the TRX_END interrupt on the PmodRF1.
    RF1WriteReg(rfrIrqMask, ( 1 << bnMaskTrxEnd ));
	/******************************************************/
		
	/* Initialize ADC to read the battery */
	// Internal 1.1V ref; ADC6 selected, Free-running, 250KHz peripheral clock
	ADMUX = (1 << REFS1)|(1 << REFS0)|(1 << MUX2)|(1 << MUX1);
	ADCSRA = (1 << ADEN)|(1 << ADIE); //ADC enabled; interrupts enabled
	ADCSRA |= (1 << ADPS1); //prescaler of 4 -> 250KHz

	sei();  //enable global interrupts	
}

/* ------------------------------------------------------------ */
/***	Wait_ms
**
**	Synopsis:
**		Wait_ms(WORD)
**
**	Parameters:
**		WORD (range from 0 to 65535)
**
**	Return Values:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Will wait for specified number of milliseconds.  Using a 
**		word variable allows for delays up to 65.535 seconds.  The value
**		in the for statement may need to be altered depending on how your
**		compiler translates this code into AVR assembly.  In assembly, it is
**		possible to generate exact delay values by counting the number of clock
**		cycles your loop takes to execute.  When writing code in C, the compiler
**		interprets the code, and translates it into assembly.  This process is 
**		notoriously inefficient and may vary between different versions of AVR Studio
**		and WinAVR GCC.  A handy method of calibrating the delay loop is to write a 
**		short program that toggles an LED on and off once per second using this 
**		function and using a watch to time how long it is actually taking to
**		complete. 
**
**		This code supports a clock of 1MHz (49).  The number in the for loop should be
**		~390 for 8MHz clock source.
**
*/

void Wait_ms(WORD delay) {

	WORD i;

	while(delay > 0){

		for( i = 0; i < 49; i ++){
			;;
		}
		delay -= 1;
	}
}

/* ------------------------------------------------------------ */
/***	RF1SendString
**
**	Synopsis:
**		BYTE RF1SendString(BYTE *szIn, BYTE cPacketLen)
**
**	Parameters:
**		szLen:		pointer to first character in a NULL terminated string
**		cPacketLen:	number of bytes in string, excluding NULL terminator
**
**	Return Values:
**		BYTE: ftrue for successful transmission
**			  fFalse for failure to transmit
**
**	Errors:
**		none
**
**	Description:
**		This function will take a NULL terminated string and send it out
**		via a PmodRF1 using functions from the PmodRF1.c library.
**
*/
BYTE RF1SendString(BYTE *szIn, BYTE cPacketLen) {

	BYTE rfrTemp;
	
	prtRF1SLPTR &= ~(1 << bnRF1SLPTR); //in SLEEP, edge sensitive, wake!
	Wait_ms(1); //only requires 380uS to wake
	
	/***** Read the channel register from the PmodRF1 ********/
	if(rfchan != ((BRF1ReadReg(rfrPhyCcCca)) & 0x1F)) {
		return fFalse; //indicate communication failure with IC
	}
	/*****************************************************/
	
	
	/**** Send out the string from the PmodRF1 **********/
	
	// Change the PmodRF1 to the PLL_ON state.
	RF1WriteReg(rfrTrxState, rfstPllOn);

	// Write the data packet to the RF1 frame buffer.
	RF1WritePkt(rfadrPARTNER, szIn, cPacketLen);

	// Wait for the PmodRF1 PLL to lock.
	rfrTemp = BRF1ReadReg(rfrIrqStatus);
	while ( ! ( rfrTemp & ( 1 << bnPllLock ) ) ) {
		rfrTemp = BRF1ReadReg(rfrIrqStatus);
	}

	// Start wireless tranmission of the packet.
	RF1WriteReg(rfrTrxState, rfcTxStart);

	// Wait for wireless packet transmission to complete.
	while ( ! (pinRF1IRQ & ( 1 << bnRF1IRQ )) ) {
		asm volatile("nop");
	}
	
	// Read and discard the IRQ status register of the PmodRF1.
	BRF1ReadReg(rfrIrqStatus);
	
	// Place the PmodRF1 in the TRX_OFF state to save power.
	RF1WriteReg(rfrTrxState, rfstTrxOff);

	// Read the PmodRF1 interrupt status register.
	BRF1ReadReg(rfrIrqStatus);
	/*******************************************************/
	
	//Put device to sleep via SLP_TR pin.  TRX_OFF : 0.4mA; SLEEP : 0.2uA!
	prtRF1SLPTR |= (1 << bnRF1SLPTR); //rising edge sensitive in TRX_OFF state. Sleep
	
	return fTrue;
}	
